'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Recursively calls toJSON on each member
 * @param {Object} obj
 */
var mapValuesToPlainObjects = function mapValuesToPlainObjects(obj) {
  return (0, _keys2.default)(obj).reduce(function (o, key) {
    o[key] = obj[key];
    if (o[key] instanceof AnemicModel) o[key] = o[key].toJSON();
    if (o[key] instanceof Object) o[key] = mapValuesToPlainObjects(o[key]);
    return o;
  }, obj instanceof Array ? [] : {});
};

/**
 * Converts anemic instance to an object with public props
 * @param {AnemicModel} obj
 */
var anemicInstanceToObject = function anemicInstanceToObject(obj) {
  return (0, _keys2.default)(obj).map(function (key) {
    return key.replace('_', '');
  }).filter(function (key) {
    return obj._properties.hasOwnProperty(key);
  }).reduce(function (result, key) {
    result[key] = obj._properties[key];
    return result;
  }, {});
};

/**
 * Class AnemicModel have the responsibility of implement the common constructor and toJSON
 * code routines and make it transparent to our consumer.
 */

var AnemicModel = function () {
  function AnemicModel() {
    var _this = this;

    var properties = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    (0, _classCallCheck3.default)(this, AnemicModel);

    (0, _keys2.default)(properties).forEach(function (key) {
      return _this['_' + key] = properties[key];
    });
    Object.defineProperty(this, '_properties', {
      value: properties,
      writable: false,
      enumerable: false,
      configurable: false
    });
  }

  /**
   * Method toJSON will get our object keys and transform it to the same structure as our input object.
   * The method will:
   * 1 - replace key _myPropertyKey by myPropertyKey
   * 2 - Check if the property was existing on the base properties constructor object or was added later. If second it will not included on te toJSON
   * 3 - Check if the property is AnemicObject and call its toJSON method too to get their JSON aspect.
   * 4 - Check if is an array, if it is will check if is an AnemicObject and do the same as step 3
   * 5 - If any of the latest steps were successful it will return the property without the dash.
   *
   * @returns {*}
   */


  (0, _createClass3.default)(AnemicModel, [{
    key: 'toJSON',
    value: function toJSON() {
      return mapValuesToPlainObjects(anemicInstanceToObject(this));
    }
  }]);
  return AnemicModel;
}();

exports.default = AnemicModel;