'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _NotImplementedUseCase = require('./NotImplementedUseCase');

var _NotImplementedUseCase2 = _interopRequireDefault(_NotImplementedUseCase);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (_ref) {
  var useCases = _ref.useCases,
      config = _ref.config;
  return function () {
    function EntryPoint() {
      var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : { config: {} };
      (0, _classCallCheck3.default)(this, EntryPoint);

      // decide to use a static config from the factory
      // or use a config passed to the constructor that could be mutated
      this._config = config || params.config;
      this._useCases = useCases;
    }

    /**
     * Retreives one entry from the public dictionary
     * The dictionary will contain the use cases and configuration
     * @example
     * const configuration = domain.get('config')
     * const useCase = domain.get('phone_contact_use_case')
     * @param {string} key The name of the use case or indicate that we want to get the config
     * @return (config|UseCase)
     */


    (0, _createClass3.default)(EntryPoint, [{
      key: 'get',
      value: function get(key) {
        var _this = this;

        // config key is still sync and we should return as it
        if (key === 'config') {
          return this._config;
        }
        // get the useCase using the key passed by the user
        var useCase = this._useCases[key];
        // if the useCase doesn't exist, then let developer know that the useCase is not implemented
        if (typeof useCase === 'undefined') {
          return new _NotImplementedUseCase2.default(key);
        }
        // if the useCase is not an array
        if (useCase instanceof Array === false) {
          return useCase;
        }
        // at this point, the developer is using webpack dynamic import for getting the useCases
        // get the loader and the method from the useCases

        var _useCase = (0, _slicedToArray3.default)(useCase, 2),
            loader = _useCase[0],
            method = _useCase[1];
        // if loader is undefined then is not implemented, otherwhise load async the useCase


        return loader === undefined ? new _NotImplementedUseCase2.default(key) : {
          execute: function execute(params) {
            // load async the factory, execute use case and return the promise
            return loader().then(function (factory) {
              return factory.default[method]({ config: _this._config }).execute(params);
            });
          },
          $: {
            execute: {
              subscribe: function subscribe(fn) {
                // creating an object here that will have a dispose method
                var ret = { dispose: function dispose() {} };
                loader().then(function (factory) {
                  // black magic: mutate the object, very small memory leak but that
                  // makes dispose working async and we need it
                  ret.dispose = factory.default[method]({
                    config: _this._config
                  }).$.execute.subscribe(fn).dispose;
                });
                // return the object that will be mutated async
                return ret;
              }
            }
          }
        };
      }

      /**
       * Set a value to a key in the config
       * @example
       * const configuration = domain.config('new_config', 'new value')
       * @param {String} key
       * @param {String} value
       * @return {Config}
       */

    }, {
      key: 'config',
      value: function config(key, value) {
        this._config.set(key, value);
        return this;
      }
    }]);
    return EntryPoint;
  }();
};